/* ===================================================================================================
* WARNING - This file is part of the base implementation for WebMaker, so it should not be edited or changed for any project.
* These files are replaced if a project is re-imported to the WebMaker Studio or migrated to a new version of the product.
* For guidance on 'How do I override or clone Hyfinity webapp files such as CSS & javascript?', please read the following relevant FAQ entry:
* http://www.hyfinity.net/faq/index.php?solution_id=1113
==================================================================================================== */

/*
 * FMActions.js
 *
 * Provides implementations of the builtin actions provided by FormMaker
 *
 * @author Hyfinity Ltd
 * Copyright (c) 2009
 * @version 1.1
 *
 */


hyf.FMAction =
{
    version: '1.1',
    desc: 'Contains all the functions that implement the set of inbuilt FormMaker actions'
}

/**
 * Performs form submission
 * @param actionParam A param object containing the name of the action to submit to in the value property.
 * @param validateParam A param object indicating whether to validate before submission.  The value property
 *                  must contain true for validation to occurr.
 */
hyf.FMAction.handleFormSubmission = function(actionParam, validateParam)
{
    var fv = hyf.FMAction.getFormValidator();
    var f = fv.getForm();
    var action = f.action;
    if (actionParam != null && (typeof(actionParam) != 'undefined'))
    {
        action = actionParam.value + '.do';
    }

    if ((validateParam != null && (typeof(validateParam) != 'undefined') &&
        (validateParam.value == true) || (validateParam.value == 'true')))
    {
        if (hyf.validation.validateForm())
        {
            f.action = action;
            if (typeof(f.onsubmit) == 'function')
            {
                var resp = f.onsubmit();
                if ((typeof(resp) == 'boolean') && (resp == false))
                    return false;
            }
            f.submit();
            return true;
        }
        else
            return false;
    }
    else
    {
        f.action = action;
        if (typeof(f.onsubmit) == 'function')
        {
            var resp = f.onsubmit();
            if ((typeof(resp) == 'boolean') && (resp == false))
                return false;
        }
        f.submit();
        return true;
    }
}

/**
 * Performs an AJAX submission.
 * @param actionParam A param object indicating which action to call
 * @param sourceParam A param object indicating which source data to send to the remote service
 * @param targetParam A param object indicating which group the ajax response should be placed into.
 * @param validateParam a param object indicating whether to first validate the source. The value must be true for validation to occurr.
 */
hyf.FMAction.handleAjaxSubmission = function(actionParam, sourceParam, targetParam, validateParam, eventSourceParam)
{
    var targetGroup = targetParam.value;
    if ((typeof(targetParam.repeatId) != 'undefined') && (targetParam.repeatId != ''))
        targetGroup = targetParam.repeatId + targetGroup;

    var sourceGroup = null;
    if (sourceParam.option != 'AllFormData')
    {
        sourceGroup = sourceParam.value;
        if ((typeof(sourceParam.repeatId) != 'undefined') && (sourceParam.repeatId != ''))
            sourceGroup = sourceParam.repeatId + sourceGroup;
    }

    var functionPrefix = '';
    if (eventSourceParam.value)
        functionPrefix = eventSourceParam.value;


    if ((validateParam.value == true) || (validateParam.value == 'true'))
        hyf.FMAction.subSectionSubmit(actionParam.value, targetGroup, sourceGroup, functionPrefix, true);
    else
        hyf.FMAction.subSectionSubmit(actionParam.value, targetGroup, sourceGroup, functionPrefix, false);
}

/**
 * Sets the value of the specified field to the indicated value.
 * @param targetParam A param object indicating the field whose value should be changed.
 * @param valueParam A param object either containing the value to set, or specifiying which
 *                   field to get the new value from.
 */
hyf.FMAction.handleSetValue = function(targetParam, valueParam, objEventSource)
{
    var targetId = targetParam.value;
    if ((typeof(targetParam.repeatId) != 'undefined') && (targetParam.repeatId != ''))
        targetId = targetParam.repeatId + targetId;

    var target = document.getElementById(targetId);
    if (target != null)
    {
        var newValue = hyf.FMAction.getParameterValue(valueParam, objEventSource);

        if (newValue != null)
        {
            hyf.util.setFieldValue(targetId, newValue);
        }
    }
}

/**
 * Returns the value that should be used from the given parameter object.
 * This handles the different types of values, eg from page field, script fragment
 * static value, etc.
 * This also deals with concating params that provide multiple values.
 * @param param The parameter object to get the value from.
 * @return The value for the param.
 */
hyf.FMAction.getParameterValue = function(param, objEventSource)
{
    var newValue = null;

    if (param.multiple)
    {
        newValue = '';
        var valArray = param.value;
        for (var i = 0; i < valArray.length; ++i)
        {
            newValue += hyf.FMAction.getValueFromParamObject(valArray[i], objEventSource);
        }
    }
    else
    {
        newValue = hyf.FMAction.getValueFromParamObject(param, objEventSource);
    }

    return newValue;
}

hyf.FMAction.getValueFromParamObject = function(obj, objEventSource)
{
    var val = '';
    if (obj.option == 'PageField')
    {
        var sourceId = obj.value;
        var getDisplayValue = false;

        if (sourceId.indexOf('**display_text') != -1)
        {
            sourceId = sourceId.substring(0, sourceId.indexOf('**display_text'));
            getDisplayValue = true;
        }

        if ((typeof(obj.repeatId) != 'undefined') && (obj.repeatId != ''))
            sourceId = obj.repeatId + sourceId;

        val = hyf.util.getFieldValue(sourceId, getDisplayValue);
    }
    else if (obj.option == 'Script')
    {
        val = dojo.hitch(objEventSource.field, function(){
                return eval(obj.value);
            })();
    }
    else if (obj.option == 'DisplayVariable')
    {
        //all display variables are stored as hidden fields on the page using a defined
        //naming format, so we need to find this field, and get its value here.
        var dvName = obj.value;
        var dvField = document.getElementById('hyf_display_variable_' + dvName);
        if (dvField != null)
            val = dvField.value;
    }
    else
        val = obj.value;

    return val;
}

/**
 * Toggles the visibility of the specified group.
 * @param groupParam A param object indicating which group should be toggled.
 * @param animateParam A param object specifying whether to animate the showing/hiding of the group.
 *                     This object's value property must be 'true' for animation to occur.
 */
hyf.FMAction.handleGroupToggle = function(groupParam, animateParam)
{
    var groupId = groupParam.value;
    if ((typeof(groupParam.repeatId) != 'undefined') && (groupParam.repeatId != ''))
        groupId = groupParam.repeatId + groupId;

    if (animateParam.value == 'true')
        hyf.util.toggleComponent(groupId, null, true);
    else
        hyf.util.toggleComponent(groupId, null, false);
}

/**
 * Evalautes the given custom script.
 * This is not used, as the script is just output directly into the generated function.
 * @param scriptParam A param object whose value property contains the javascript string to evalaute.
 */
hyf.FMAction.handleCustomScript = function(scriptParam)
{
    eval(scriptParam.value);
}

/**
 * Resets the form controls in the specific container.
 * @param containerParam A param object indicating the container whose controls should be reset.
 * @param modeParam A param object specifying which type of reset to perform.  The option property
 *                      of this object should be either 'reset' or 'clear'
 */
hyf.FMAction.handleContainerReset = function(containerParam, modeParam)
{
    var container = null;
    if (containerParam.option == 'Form')
        container = hyf.FMAction.getFormValidator().getForm();
    else
    {
        var contId = containerParam.value;
        if ((typeof(containerParam.repeatId) != 'undefined') && (containerParam.repeatId != ''))
            contId = containerParam.repeatId + contId;

        container = document.getElementById(contId);
    }

    hyf.util.resetContainer(container, modeParam.option);
}


/**
 * Displays an alert message with the specified content.
 * @param messageParam A param object indicating the message text to display, or the
 *          script to evalaute to get the message.
 */
hyf.FMAction.handleAlert = function(messageParam, objEventSource)
{
    var msg = hyf.FMAction.getParameterValue(messageParam, objEventSource);
    alert(msg);
}

/**
 * Handles calling a specified function, with a number of parameters
 * @param nameParam Parameter object whose value should contain the name of the function to call.
 * @param paramsParam Parameter object containing all the params to pass to the funciton when called.
 * @return The response from the function.
 */
hyf.FMAction.handleFunctionCall = function(nameParam, paramsParam, objEventSource)
{
    var funcName = nameParam.value;
    var fn = eval(funcName);
    if (typeof(fn) == 'function')
    {
        var functionParams = new Array();
        if (paramsParam.multiple)
        {
            var valArray = paramsParam.value;
            for (var i = 0; i < valArray.length; ++i)
            {
                functionParams[functionParams.length] = hyf.FMAction.getValueFromParamObject(valArray[i], objEventSource);
            }
        }
        else
        {
            if (paramsParam.option != '')
                functionParams[functionParams.length] = hyf.FMAction.getValueFromParamObject(paramsParam, objEventSource);
        }

        return fn.apply(this, functionParams);
    }
}

/**
 * Package for the funcitonality related to raise custom errors
 */
hyf.FMAction.raiseError = {};
/**
 * Event handler for the raise error action.
 * This creates a new valdiation error for the specified field, and initialises the custom
 * message produce function so that the required message will be displayed.
 * @param errorFieldParam The param object indicating which field to raise the error against.
 * @param messageParam The param object indicating the custom error message to display
 */
hyf.FMAction.raiseError.handle = function(errorFieldParam, messageParam, objEventSource)
{
    var targetId = errorFieldParam.value;
    if ((typeof(errorFieldParam.repeatId) != 'undefined') && (errorFieldParam.repeatId != ''))
        targetId = errorFieldParam.repeatId + targetId;

    var target = document.getElementById(targetId);
    if (target != null)
    {
        //check if our custom message handler has already been registered
        if (!hyf.FMAction.raiseError.producerRegistered)
        {
            hyf.validation.DisplayMessages.addMessageProducer(hyf.FMAction.raiseError.messageProducer);
            hyf.FMAction.raiseError.producerRegistered = true;
        }

        //store the message details for future use.
        target._raiseErrorMsg = hyf.FMAction.getValueFromParamObject(messageParam, objEventSource);

        //create and register the error
        var newError = new hyf.validation.ValidationError(target, hyf.FMAction.raiseError.code);
        hyf.FMAction.getErrorDisplay().addError(newError);
    }
}

//The code used for validation errors created for custom errors
hyf.FMAction.raiseError.code = 80;
//Boolean flag used to keep track of wether or not the custom message producer function has been registered yet
hyf.FMAction.raiseError.producerRegistered = false;

/**
 * Custom message producer function for the raise error action.
 * This checks if the provided error is for the correct code,
 * and then looks for a raise error message on the field.
 * If found, this is returned as the display message to use in the valdiation process.
 */
hyf.FMAction.raiseError.messageProducer = function(field, errorCode)
{
    if (errorCode == hyf.FMAction.raiseError.code)
    {
        if (typeof(field._raiseErrorMsg != 'undefined'))
        {
            return '' + field._raiseErrorMsg;
        }
    }
}


/**
 * Handler for the change style FM action.  This supports adding, removing or toggling a CSS
 * class name on a specific component.
 * @param targetParam The param object detailing the target to chaneg the class on.
 * @param modeParam The option property of this param object indicates which type of action to perform,
 *                      'add', 'remove', or 'toggle'
 * @param valueParam This provides the name of the CSS class
 */
hyf.FMAction.handleChangeStyle = function(targetParam, modeParam, valueParam, objEventSource)
{
    var targets = [];

    if (targetParam.option == 'CSSSelector')
        targets = dojo.query(targetParam.value)
    else
    {
        var targetId = targetParam.value;
        if ((typeof(targetParam.repeatId) != 'undefined') && (targetParam.repeatId != ''))
            targetId = targetParam.repeatId + targetId;

        if (document.getElementById(targetId))
            targets = new dojo.NodeList(document.getElementById(targetId));
    }

    if (targets.length > 0)
    {
        var className = hyf.FMAction.getParameterValue(valueParam, objEventSource);

        switch (modeParam.option)
        {
            case 'add'  :   targets.addClass(className); break;
            case 'remove':  targets.removeClass(className); break;
            case 'toggle':  targets.toggleClass(className); break;
        }
    }
}

/**
 * Sets the text content of the label for the provided field.
 * @param targetParam A param object indicating the field whose label value should be changed.
 * @param valueParam A param object specifying the value to use for the label.
 */
hyf.FMAction.handleSetLabel = function(targetParam, valueParam, objEventSource)
{
    var targetId = targetParam.value + '_label';
    if ((typeof(targetParam.repeatId) != 'undefined') && (targetParam.repeatId != ''))
        targetId = targetParam.repeatId + targetId;

    var target = document.getElementById(targetId);

    //if target is null, try without the repeat information for the situation where we are changing
    //the label of a field in a table.
    if ((target == null) && (typeof(targetParam.repeatId) != 'undefined') && (targetParam.repeatId != ''))
    {
        targetId = targetParam.value + '_label';
        target = document.getElementById(targetId);
    }

    if (target != null)
    {
        var newValue = hyf.FMAction.getParameterValue(valueParam, objEventSource);

        if (newValue != null)
        {
            hyf.util.setFieldValue(targetId, newValue);
        }
    }
}

/**
 * Updates the value of the specified display variable.
 * @param targetParam A param object indicating the display variable to update.
 * @param valueParam A param object specifying the new value to use.
 */
hyf.FMAction.handleSetDisplayVariable = function(targetParam, valueParam, objEventSource)
{
    var dvName = targetParam.value;
    var dvField = document.getElementById('hyf_display_variable_' + dvName);
    if (dvField != null)
    {
        var newValue = hyf.FMAction.getParameterValue(valueParam, objEventSource);

        dvField.value = newValue;
    }
}

/**
 * Generates the return value that should be used for the terminate action.
 * @param valueParam the param object indicating the return value.
 */
hyf.FMAction.handleTerminate = function(valueParam, objEventSource)
{
    var returnValue = hyf.FMAction.getParameterValue(valueParam, objEventSource);

    //convert true or false as a string to the actual boolean values.
    if (returnValue == 'true')
        return true;
    else if (returnValue == 'false')
        return false;
    return returnValue;
}

/**
 * Implements the calculation action which can calculate the addition/subtraction/multiplication/division of a number of values
 * and place the result into either a field or display variable.
 * @param targetParam Param object inidicating where the result of the calculation should be placed (either field or display variable)
 * @param operatorParam Param object indicating which operation should be performed. This will be determined by the option value of the object
 * @param valuesParam Param object specifying all the values to process.
 * @param formatParam Param specifying the format to apply to the resulting value (optional)
 * @param objEventSource
 */
hyf.FMAction.handleCalculation = function(targetParam, operatorParam, valuesParam, formatParam, objEventSource)
{
    var sourceValues = hyf.FMAction.handleCalculationFindValues(valuesParam, objEventSource);
    var op = operatorParam.option;

    var result = Number(sourceValues[0]);

    for( var i = 1; i < sourceValues.length; ++i)
    {
        switch (op)
        {
            case '+' : result += Number(sourceValues[i]); break;
            case '-' : result -= Number(sourceValues[i]); break;
            case '*' : result = result * Number(sourceValues[i]); break;
            case '/' : result = result / Number(sourceValues[i]); break;
        }
    }

    //check if any format pattern has been defined
    if (formatParam.option != '')
    {
        result = dojo.number.format(result, {pattern: formatParam.value});
    }

    //check whether to update a field or display variable
    if (targetParam.option == 'PageField')
    {
        var targetId = targetParam.value;
        if ((typeof(targetParam.repeatId) != 'undefined') && (targetParam.repeatId != ''))
            targetId = targetParam.repeatId + targetId;
        hyf.util.setFieldValue(targetId, result);
    }
    else if (targetParam.option == 'DisplayVariable')
    {
        var dvName = targetParam.value;
        var dvField = document.getElementById('hyf_display_variable_' + dvName);
        if (dvField != null)
        {
            dvField.value = result;
        }
    }
}

/**
 * Override of the noraml getParameterValue method for the calculation action so that it returns an array of all the found values,
 * rather than a single concat string
 */
hyf.FMAction.handleCalculationFindValues = function(valuesParam, objEventSource)
{
    var valuesArray = new Array();

    if (valuesParam.multiple)
    {
        var valArray = valuesParam.value;
        for (var i = 0; i < valArray.length; ++i)
        {
            valuesArray = valuesArray.concat(hyf.FMAction.handleCalculationGetValueFromParamObject(valArray[i], objEventSource));
        }
    }
    else
    {
        valuesArray = valuesArray.concat(hyf.FMAction.handleCalculationGetValueFromParamObject(valuesParam, objEventSource))
    }

    return valuesArray;
}

/**
 * Overrride the normal getValueFromParamObject for the calculation action so that we can handle the
 * 'all_across_repeat' option to return an array of the values for the field in each repeat entry.
 */
hyf.FMAction.handleCalculationGetValueFromParamObject = function(obj, objEventSource)
{
    var values = new Array();
    if ((obj.option == 'PageField') && (obj.value.indexOf('**all_across_repeat') != -1))
    {
        var sourceId = obj.value;
        sourceId = sourceId.substring(0, sourceId.indexOf('**all_across_repeat'));

        //remove the ending row identifier from the repeat id
        var marker = obj.repeatId.lastIndexOf(obj.repeatName);
        var repId = obj.repeatId.substring(0, marker + obj.repeatName.length);

        var i = 1;
        var valPart = hyf.util.getFieldValue(repId + i + sourceId);
        while (valPart != null)
        {
            values[values.length] = valPart;
            ++i;
            valPart = hyf.util.getFieldValue(repId + i + sourceId);
        }
    }
    else
        values[values.length] = hyf.FMAction.getValueFromParamObject(obj, objEventSource);

    return values;
}





/**
 * Returns the form validator object that should be used for all validation
 * functionality.
 * This currently assumes that there is only one form on the page that should be validated.
 * If needed, we can extend this to allow selection of which form should be validated, and
 * so return the correct validator object.
 * @return The hyf.validation.FormValidator object to use for validation.
 * @private
 */
hyf.FMAction.getFormValidator = function()
{
    if (typeof(hyf.FMAction.formValidator) == 'undefined')
    {
        var f;
        if (hyf.validation.config.form)
            f = hyf.validation.config.form;
        else
            f = document.forms[0];

        hyf.FMAction.formValidator = new hyf.validation.FormValidator(f);
    }

    return hyf.FMAction.formValidator;
}

/**
 * Returns the error display object that should be used for displaying any
 * validation errors
 * @return The hyf.validation.ErrorDisplay object to use for showing errors.
 * @private
 */
hyf.FMAction.getErrorDisplay = function()
{
    if (typeof(hyf.FMAction.errorDisplay) == 'undefined')
    {
        var dispMethod = 'none'
        if (typeof(hyf.validation.config.errorDisplayMethod) != 'undefined')
            dispMethod = hyf.validation.config.errorDisplayMethod;

        hyf.FMAction.errorDisplay = new hyf.validation.ErrorDisplay(dispMethod);

        if (typeof(hyf.validation.config.errorDisplayFieldStyle) != 'undefined')
            hyf.FMAction.errorDisplay.setFieldStyle(hyf.validation.config.errorDisplayFieldStyle);
        if (typeof(hyf.validation.config.errorDisplayFieldClass) != 'undefined')
            hyf.FMAction.errorDisplay.setFieldClass(hyf.validation.config.errorDisplayFieldClass);
        if (typeof(hyf.validation.config.errorDisplayMessageStyle) != 'undefined')
            hyf.FMAction.errorDisplay.setMessageStyle(hyf.validation.config.errorDisplayMessageStyle);
        if (typeof(hyf.validation.config.errorDisplayMessageClass) != 'undefined')
            hyf.FMAction.errorDisplay.setMessageClass(hyf.validation.config.errorDisplayMessageClass);
        if (typeof(hyf.validation.config.errorDisplayShowMessage) != 'undefined')
            hyf.FMAction.errorDisplay.setShowMessage(hyf.validation.config.errorDisplayShowMessage);
        if (typeof(hyf.validation.config.errorDisplayShowAlerts) != 'undefined')
            hyf.FMAction.errorDisplay.setShowAlerts(hyf.validation.config.errorDisplayShowAlerts);
        if (typeof(hyf.validation.config.errorDisplayValidationMode) != 'undefined')
            hyf.FMAction.errorDisplay.setValidationMode(hyf.validation.config.errorDisplayValidationMode);
        if (typeof(hyf.validation.config.errorDisplayMessageLocation) != 'undefined')
            hyf.FMAction.errorDisplay.setMessageLocation(hyf.validation.config.errorDisplayMessageLocation);
        if (typeof(hyf.validation.config.errorDisplayMessageString) != 'undefined')
            hyf.FMAction.errorDisplay.setMessageString(hyf.validation.config.errorDisplayMessageString);
    }

    return hyf.FMAction.errorDisplay;

}



/**
 * Main function used to handle any sub section submission functionality.
 * This is used to update part of the page by making an asynchronous call to the server for more data.
 *
 * @param URL The URL to call to retrieve the new information.
 * @param target The ID of the container in which to place the resulting HTML retrieved.
 * @param source (optional) The ID of a source container.  The contents of any input fields within this container
 *                  will be sent in the request to the remote service.
 *                  If this parameter is not present, the full details on the page will be submitted.
 * @param functionPrefix (optional) A string containing the prefix that will be used for all custom functions
 *                  related to this sub section submission.  The process will check for the presense of
 *                  three different functions that will be called if they exist:
 *                      <functionPrefix>ConfigureRequestParameters(requestParams)
 *                      <functionPrefix>SetLoadingMessage(message)
 *                      <functionPrefix>ManipulateResponse(response, successful)
 * @param validate (optional) A boolean field to indicate whether or not the data being submitted should be
 *                  first validated for compliance with the defined constraints.
 *                  If not provided, no validation will be performed.
 * @return A boolean value indicating whether or not the asynchronous call has been initiated.
 */
hyf.FMAction.subSectionSubmit = function(URL, target, source, functionPrefix, validate)
{
    if (typeof(dojo) == 'undefined')
    {
        alert('This functionality requires the dojo framework to be available');
        return false;
    }

    //check if we need to validate the data being sent
    if (validate)
    {
        //check if we need to validate a group or the whole page
        if ((typeof(source) == 'undefined') || (source == null) || (document.getElementById(source) == null))
        {
            if (!hyf.validation.validateForm())
                return false;
        }
        else
        {
            if (!hyf.validation.validateContainer(document.getElementById(source)))
                return false;
        }
    }

    var content;

    if ((typeof(source) == 'undefined') || source == null || document.getElementById(source) == null)
    {
        content = dojo.formToObject(hyf.FMAction.getFormValidator().getForm());
    }
    else
    {
        if (URL.indexOf('.do') != -1)
            content = hyf.util.encodeContainer(document.getElementById(source), URL.substring(0, URL.length - 3));
        else
            content = hyf.util.encodeContainer(document.getElementById(source));
    }

    var requestArgs = {
            url         :   URL,
            content     :   content,
            mimetype    :   "text/html",
            handleAs    :   "text"
            };

    //Add a hook for manipulation of the request args
    if (eval("window." + functionPrefix + 'ConfigureRequestParameters'))
        eval(functionPrefix + 'ConfigureRequestParameters(requestArgs)');

    requestArgs.handle = hyf.FMAction.subSectionHandler;
    requestArgs.hyfTarget = target;
    requestArgs.hyfFunctionPrefix = functionPrefix;

    //Display a loading message in the target container
    var loadingContent = '<span class="ajaxLoading">Loading...</span>';
    //check if the user has created a function to override this message
    if (eval("window." + functionPrefix + 'SetLoadingMessage'))
        eval('loadingContent = ' + functionPrefix + 'SetLoadingMessage(loadingContent)');

    //if the message is empty, don't change the display
    if (loadingContent != null)
        hyf.util.insertContent(document.getElementById(target), loadingContent);

    // dojo.io.bind(requestArgs); ash_change
    dojo.xhrPost(requestArgs);

    return true;
};



/**
 * Handler function called after each ajax call.
 * This is the method syntax required by dojo
 *
 * @param response The response object retreived.
 * @param ioArgs The request parameters array containing all the details used to inititate the request
 *
 * @private
 */
hyf.FMAction.subSectionHandler = function(response, ioArgs)
{

    var target = document.getElementById(ioArgs.args.hyfTarget);

    if(response instanceof Error)
    {
        //QUESTION: Do we want to pass the ioArgs object to the function?
        var content = "Unable to show details.  Please try again.";
        if (eval("window." + ioArgs.args.hyfFunctionPrefix + 'ManipulateResponse'))
            eval('content = ' + ioArgs.args.hyfFunctionPrefix + 'ManipulateResponse(content, false, response, ioArgs)');

        hyf.util.insertContent(target, content, true);
    }
    else
    {
        var content = response;
        //Add hook for manipulation of the response here (eg client side XSL)
        if (eval("window." + ioArgs.args.hyfFunctionPrefix + 'ManipulateResponse'))
            eval('content = ' + ioArgs.args.hyfFunctionPrefix + 'ManipulateResponse(content, true)');

        hyf.util.insertContent(target, content, true);
    }
};




hyf.FMCondition =
{
    version: '1.0',
    desc: 'Contains all the functions used to provide the inbuilt FormMaker event conditions'
}

/**
 * Validates the whole form, displaying any error found.
 * @return boolean indicating whether or not the valdiation was successful
 */
hyf.FMCondition.checkFormValdiation = function()
{
    return hyf.validation.validateForm();
}

/**
 * Validates the specified group, displaying any error found.
 * @param groupParam A param object specifying which group to try and validate
 * @return boolean indicating whether or not the valdiation was successful
 */
hyf.FMCondition.checkGroupValdiation = function(groupParam)
{
    var groupName = groupParam.value;
    if ((typeof(groupParam.repeatId) != 'undefined') && (groupParam.repeatId != ''))
        groupName = groupParam.repeatId + groupName;

    var group = document.getElementById(groupName);
    if (group != null)
        return hyf.validation.validateContainer(group);
    else
        return false;
}

/**
 * Displays a confirmation message to the user, containing the specified message
 * @param messageParam A param object specifying the message text
 * @return boolean indicating whether the user accepted or not.
 */
hyf.FMCondition.checkConfirm = function(messageParam, objEventSource)
{
    var msg = hyf.FMAction.getParameterValue(messageParam, objEventSource);

    return confirm(msg);
}

/**
 * Compares the given fields value to the reference, and returns the boolean result.
 * @param sourceParam A param object specifyinf the fields whose value should be compared.
 * @param comparisonParam A param object indicating the type of comparison (eg =, !=, <, >,)
 * @param referenceParam A param object indicating which reference value to use.
 * @return Boolean indicating whether the comparison was true or not.
 */
hyf.FMCondition.checkFieldValue = function(sourceParam, comparisonParam, referenceParam, objEventSource)
{
    var option = sourceParam.option;
    var sourceField = null;
    var sourceValue = null;

    if (option == 'PageField')
    {
        var sourceId = sourceParam.value;
        if ((typeof(sourceParam.repeatId) != 'undefined') && (sourceParam.repeatId != ''))
            sourceId = sourceParam.repeatId + sourceId;

        sourceField = document.getElementById(sourceId);
        if (sourceField != null)
        {
            sourceValue = hyf.util.getFieldValue(sourceId);
        }
    }
    else if (option == 'DisplayVariable')
    {
        var sourceField = document.getElementById('hyf_display_variable_' + sourceParam.value);
        if (sourceField != null)
        {
            sourceValue = sourceField.value;
        }
    }

    if (sourceValue != null)
    {
        var refValue = hyf.FMAction.getParameterValue(referenceParam, objEventSource);

        if (refValue != null)
        {
            var dataType = sourceField.getAttribute('_type');
            if (dataType == 'date')
            {
                //convert both values in the data format if possible
                var convertedValue = hyf.validation.ValueConverter.performDateConversion(sourceField, sourceValue);
                var compareResult;

                //handle the special case where the ref value is actually from another date field, and if
                //so we need to convert the value based on that fields details
                if (!referenceParam.multiple && referenceParam.option == 'PageField')
                {

                    var refFieldId = referenceParam.value;
                    if ((typeof(referenceParam.repeatId) != 'undefined') && (referenceParam.repeatId != ''))
                        refFieldId = referenceParam.repeatId + refFieldId;
                    var refField = document.getElementById(refFieldId);
                    refValue = hyf.validation.ValueConverter.performDateConversion(refField, refValue);
                    compareResult = compareDates(convertedValue, sourceField.getAttribute("_data_date_format"), refValue, refField.getAttribute("_data_date_format"));
                }
                else
                {
                    compareResult = compareDates(convertedValue, sourceField.getAttribute("_data_date_format"), refValue, sourceField.getAttribute("_data_date_format"));
                }

                //compareResult will be -1, 0, 1
                if (compareResult == 'error')
                    return false;

                switch (comparisonParam.option)
                {
                    case '=='   :   return (compareResult == 0);
                    case '!='   :   return (compareResult != 0);
                    case '<'    :   return (compareResult == -1);
                    case '>'    :   return (compareResult == 1);
                    case '<='   :   return (compareResult < 1);
                    case '>='   :   return (compareResult > -1);
                    case 'contains' : return false;
                }
            }
            else
            {
                //QUESTION: Do we need explicit support for any other data types?
                if (comparisonParam.option == 'contains')
                    return (sourceValue.indexOf(refValue) != -1)
                else
                    return eval('sourceValue ' + comparisonParam.option + ' refValue');
            }
        }
    }
    return false;
}






//------------------------------------------------------------------------------------------------
// BizFlow specific events functionality
// QUESTION: Should this be put into a seperate file?
hyf.HSGAction = {};


hyf.HSGAction.handleWIHAction = function(actionParam, responseParam)
{
    var functionName = actionParam.value;

    if (functionName != '')
    {
        try
        {
            var scriptToEval = 'basicWIHActionClient.' + functionName + '()';
            eval(scriptToEval);
        }
        catch (e)
        {
            //we ignore errors here, as the most likely cause is that the page is not actually
            //being run within the WIH environment.
        }
    }
}

hyf.HSGAction.handleWIHFlowControl = function(controlParam)
{
    var flowControl = controlParam.option;

    if ((flowControl != null) && (flowControl != ''))
    {
        try
        {
            if (flowControl == 'stop')
                basicWIHActionClient.setStop();
            else if (flowControl == 'wait')
                basicWIHActionClient.setWait();
            else if (flowControl == 'continue')
                basicWIHActionClient.setContinue();
        }
        catch (e)
        {
            //we ignore errors here, as the most likely cause is that the page is not actually
            //being run within the WIH environment.
        }
    }
}
