/**
 * -----------------------------------------------------------------------------------
 * Hyfinity - Convert Repeat Table to jQuery Visualiser Table
 * Author: Hyfinity Limited
 * Copyright (c) 2010 Hyfinity Limited
 * -----------------------------------------------------------------------------------
*/


if (typeof(hyf) == 'undefined')
{
    hyf = {
        version: 1.0
    }
}


hyf.chart = {
    version: 1.0,
    desc: 'Provides functionality for showing charts from tables of data.'
}

/**
 * Converts a standard repeat table so that it can be used by the jQuery Visualiser
 * plugin to show a chart.
 * @param table The HTML table object to convert
 */
hyf.chart.prepareChart = function(table)
{
    //check if the table has already been converted
    if ($(table).find('caption').length == 0)
    {
        //try and add a caption to the table
        var headingsId = $('thead > tr', table).get(0).getAttribute('id');
        var tableLabel = $('#' + headingsId.substr(0, headingsId.length - 9) + '_label').text();
        $('<caption>' + tableLabel + '</caption>').prependTo(table);

        var labelClass = ''

        //convert first header to td, and remove nested content from each th
        $('thead > tr > th', table).each(function(index){
                var newElem;
                if (index == 0)
                {
                    labelClass = this.className + ' ' + $(this).find('span').get(0).className;
                    newElem = $('<td class="'+labelClass+'">' + $(this).text() + '</td>');
                }
                else
                    newElem = $('<th class="'+labelClass+'">' + $(this).text() + '</th>');

                newElem[0].setAttribute('id', this.getAttribute('id'));
                $(this).replaceWith(newElem);
            });

        //convert all first column tds to ths with scope=row, and remove nested spans
        $('tbody > tr > td:first-child', table).each(function(){
                $(this).replaceWith('<th class="'+labelClass+'" scope="row">' + $(this).text() + '</th>');
            });
    }
}

/**
 * Renders a chart for the given table, using the options provided.
 * This uses the jQuery Visualiser plugin
 * @param tableId The id of the HTML table containing the data to show in the chart
 * @param chartLocationId The id of the div in which to display the chart
 * @param options The set of options controlling how the chart will be displayed.
 */
hyf.chart.renderChart = function(tableId, chartLocationId, options)
{
    if (hyf.pageLoaded)
    {
        var table = document.getElementById(tableId);
        var chartLocation = document.getElementById(chartLocationId);
        //ensure the table has been converted as needed
        hyf.chart.prepareChart(table);

        if (chartLocation != null)
        {
            chartLocation = $(chartLocation);
            chartLocation.addClass('visualize');
        }

        //now render the chart using the provided options
        $(table).visualize(options, chartLocation);
    }
    else
    {
        hyf.attachEventHandler(window, 'onload', function() {
                hyf.chart.renderChart(tableId, chartLocationId, options);
            });
    }
}

/**
 * Finds the last table object that has been output to the document.
 * This is used as the page is being rendered to initialise any charts
 */
hyf.chart.getLastTableOutput = function()
{
    var tables = document.getElementsByTagName('table');

    return tables[tables.length - 1];
}

